/**************************************************************************/
/*                                                                        */
/* Copyright (c) 2005 NoMachine, http://www.nomachine.com.                */
/*                                                                        */
/* NXKILL, NX protocol compression and NX extensions to this software     */
/* are copyright of NoMachine. Redistribution and use of the present      */
/* software is allowed according to terms specified in the file LICENSE   */
/* which comes in the source distribution.                                */
/*                                                                        */
/* Check http://www.nomachine.com/licensing.html for applicability.       */
/*                                                                        */
/* NX and NoMachine are trademarks of Medialogic S.p.A.                   */
/*                                                                        */
/* All rigths reserved.                                                   */
/*                                                                        */
/**************************************************************************/

#include "Control.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>
#include <signal.h>
#include <dirent.h>

#define INDEX_LIST    0
#define INDEX_KILL    1
#define INDEX_ID      2
#define INDEX_PID     3
#define INDEX_HARD    4
#define INDEX_SIGNAL  5
#define INDEX_HELP    6
#define INDEX_VERSION 7

//
// Options Allowed in nxkill application
//

static struct option OPTIONS[] = {
  {"list", 0, 0, 0},
  {"kill", 0, 0, 0},
  {"id", 1, 0, 0},
  {"pid", 1, 0, 0},
  {"hard", 0, 0, 0},
  {"signal",1, 0, 0},
  {"help", 0, 0, 0},
  {"version", 0, 0, 0},
  {0, 0, 0, 0}
};

#undef DEBUG
#undef TEST
#undef WARNING

Control::Control()

  : listMode_(false), killMode_(false), killHard_(false),
    id_(NULL), pid_(-1),signal_(SIGTERM),nxDirPath_(NULL)
{

}

Control::~Control()
{
  if (id_ != NULL) delete(id_);
}

void Control::printHelp()
{
  printf("Usage: nxkill [OPTIONS]\n\n");
  printf("--list [--id SESSIONID]\n");
  printf("  If given without a further parameter, it lists the running\n");
  printf("  sessions. If given with a session id, it lists the PIDs of the\n");
  printf("  processes that are run as part of the session.\n\n");
  printf("  Example:\n\n");
  printf("  # nxkill --list\n");
  printf("  S test00 1016 6C84D04D743CEB0A71D6EF9AE4B967E3\n");
  printf("  C test00 1016 6C84D04D743CEB0A71D6EF9AE4B967E3\n");
  printf("  S saturno.medialogic.it 1024 812312312EAECE1231123123210000\n\n");
  printf("  Or:\n\n");
  printf("  # nxkill --list --id 6C84D04D743CEB0A71D6EF9AE4B967E3\n");
  printf("  P 12344\n");
  printf("  X 3211\n\n");
  printf("  P stands for proxy, X for X server.\n\n");
  printf("--id\n");
  printf("  Allows for giving the ID of the session on which to execute the\n");
  printf("  command.\n\n");
  printf("--kill\n\n");
  printf("  Kills the process or the session with the default signal.\n\n");
  printf("  Example:\n\n");
  printf("  # nxkill --kill --pid 123321\n");
  printf("  # nxkill --kill --id 6C84D04D743CEB0A71D6EF9AE4B967E3\n\n");
  printf("--pid\n\n");
  printf("  Allows for giving the PID of the process.\n\n");
  printf("--hard\n\n");
  printf("  If given in a --kill, kills the process or the session hard,\n");
  printf("  trying its best to ensure that the target is terminated before\n");
  printf("  giving up. If it can't, times out after 5 retries and 20 secs.\n");
  printf("  and prints an error.\n\n");
  printf("--signal\n\n");
  printf("  Specify the signal by number.\n\n");
  printf("  Example:\n\n");
  printf("  # nxkill --kill --pid 1234 --signal 2\n\n");
  printf("--help\n\n");
  printf("  Display this help end exit.\n\n");
  printf("--version\n\n");
  printf("  Print version and exit.\n\n");
}

void Control::printVersion()
{
  printf("NXKILL - Version %s\n",VERSION);
}

int Control::parseCommandLineOptions(int argc, char** argv)
{
  int c;

  while (1)
  {
    int option_index = 0;

    // This doesn't compile on cygwin: c = getopt_long_only(argc, argv, "", OPTIONS, &option_index);
    c = getopt_long(argc, argv, "", OPTIONS, &option_index);

    if (c == -1)
    {
      break;
    }

    if (c == '?')
    {
      printf("ERROR: unknown option!\n");
      return -1;
    }

    #ifdef DEBUG

    printf("option '%s' ",OPTIONS[option_index].name);

    if (optarg)
    {
      printf("with arg '%s'",optarg);
    }

    printf("\n");

    #endif

    switch (option_index)
    {
    case INDEX_LIST:

      listMode_ = true;

      break;

    case INDEX_KILL:

      killMode_ = true;
      break;

    case INDEX_ID:

      if (listMode_ || killMode_)
      {
        id_ = (char *)strdup(optarg);
      }
      else
      {
        return -1;
      }

      break;

    case INDEX_PID:

      if (listMode_ || killMode_)
      {
        //
        // FIXME: problem when passing string instead of number
        //        atoi returns 0
        //
        pid_ = atoi(optarg);
      }

      break;

    case INDEX_HARD:

      if (killMode_)
      {
        killHard_ = true;
      }
      break;

    case INDEX_SIGNAL:

      if (killMode_)
      {
        signal_ = atoi(optarg);
      }
      break;

    case INDEX_HELP:

      printHelp();
      return 1;
      break;

    case INDEX_VERSION:
      printVersion();
      return 1;
      break;

    default:
      printf("ERROR: unknown option!\n");
      return -1;
    }
  }

  //
  // When we are in KILL mode we have to have ID or PID
  // specified (both is wrong).
  //

  if ((killMode_ && !((id_ == NULL) ^ (pid_ == -1))))
  {
    printf("ERROR: You should specify PID or ID when killing.\n");
    return -1;
  }

  char *homePath = NULL;

  if ((homePath = getenv("NX_HOME")) != NULL)
  {

    nxDirPath_ = homePath;

    DIR *dir = opendir(homePath);

    if (dir == NULL)
    {
      printf("Error: Cannot open directory pointed by NX_HOME = '%s'. Exiting.\n",nxDirPath_);
      return -1;
    }

    closedir(dir);

  }
  else if ((homePath = getenv("HOME")) != NULL)
  {
    nxDirPath_ = new char[strlen(homePath) + strlen("/.nx") + 1];

    strcpy(nxDirPath_,homePath);
    strcat(nxDirPath_,"/.nx");

    DIR *dir = opendir(nxDirPath_);

    if (dir == NULL)
    {
      printf("Error: Cannot open NX directory '%s'. Exiting.\n",nxDirPath_);
      return -1;
    }

    closedir(dir);
  }
  else
  {
    printf("ERROR: NX_HOME or HOME variable are not set. Exiting.\n");
    return -1;
  }

  #ifdef DEBUG
  printf("Debug: nxdir = [%s].\n",nxDirPath_);
  #endif

  return 0;
}


bool Control::isListMode() const
{
  return listMode_;
}

bool Control::isKillMode() const
{
  return killMode_;
}

bool Control::isKillHard() const
{
  return killHard_;
}

char* Control::getSessionID()
{
  return id_;
}

int Control::getPid() const
{
  return pid_;
}

int Control::getSignal() const
{
  return signal_;
}

char* Control::getDotNXDirPath()
{
  return nxDirPath_;
}
